﻿
using System;
using UAOOI.SemanticData.UANodeSetValidation.DataSerialization;

namespace UAOOI.Networking.Simulator.Boiler.AddressSpace
{
  public class NodeState : IDisposable
  {

    #region constructors
    /// <summary>
    /// Creates an empty object.
    /// </summary>
    /// <param name="nodeClass">The node class.</param>
    protected NodeState(NodeClass nodeClass, QualifiedName browseName)
    {
      NodeClass = nodeClass;
      m_browseName = browseName;
    }
    [Obsolete("This constructor is provided only to make auto-generated code error free")]
    public NodeState(NodeState parent)
    {
      throw new NotImplementedException("Autogenerated constructor cannot be used.");
    }
    #endregion

    /// <summary>
    /// The browse name of the node.
    /// </summary>
    /// <value>The name qualified with a namespace.</value>
    internal QualifiedName BrowseName
    {
      get
      {
        return m_browseName;
      }
      set
      {
        if (!Object.ReferenceEquals(m_browseName, value))
        {
          ChangeMasks |= NodeStateChangeMasks.NonValue;
        }

        m_browseName = value;
      }
    }
    internal protected NodeStateChangeMasks ChangeMasks { get; protected set; }
    internal NodeClass NodeClass { get; private set; }

    #region IDisposable Support
    private bool disposedValue = false; // To detect redundant calls
    protected virtual void Dispose(bool disposing)
    {
      if (!disposedValue)
      {
        if (disposing)
        {
          // TODO: dispose managed state (managed objects).
        }
        // TODO: free unmanaged resources (unmanaged objects) and override a finalizer below.
        // TODO: set large fields to null.
        disposedValue = true;
      }
    }
    // This code added to correctly implement the disposable pattern.
    public void Dispose()
    {
      // Do not change this code. Put cleanup code in Dispose(bool disposing) above.
      Dispose(true);
    }
    #endregion

    #region private
    private QualifiedName m_browseName;
    #endregion

  }

  /// <summary>
  /// Indicates what has changed in a node.
  /// </summary>
  public enum NodeStateChangeMasks
  {
    /// <summary>
    /// None has changed
    /// </summary>
    None = 0x00,

    /// <summary>
    /// One or more children have been added, removed or replaced.
    /// </summary>
    Children = 0x01,

    /// <summary>
    /// One or more references have been added or removed.
    /// </summary>
    References = 0x02,

    /// <summary>
    /// The value attribute has changed.
    /// </summary>
    Value = 0x04,

    /// <summary>
    /// One or more non-value attribute has changed.
    /// </summary>
    NonValue = 0x08,

    /// <summary>
    /// The node has been deleted.
    /// </summary>
    Deleted = 0x10
  }

}