# Object Oriented Internet

* [Executive Summary](README.MD)
* [Table of Content](TableOfContent.md)

## Introduction

* [Introduction to Object-Oriented Internet](Preface.md)
* [Introduction to Complex Data Processing](Introduction2ComplexDataProcessing.md)
* [OPC Unified Architecture](OPCUnifiedArchitecture.md)
* [OPC UA Main Technology Features](OPCUAMainTechnologyFeatures.md)

## Semantic-Data Processing

* [Semantic-Data Processing Architecture](SemanticData/README.MD)
* [Address Space and Address Space Model](SemanticData/AddressSpaceAddressSpaceModel.md)
* [UA Information Model - Concept](SemanticData/InformationModelConcept.md)
  * [Standard Information Model](SemanticData/StandardInformationModel.md)
* [Information Models Development](SemanticData/InformationModelsDevelopment.md)
  * [Adopting Companion Standard Models - Analyzer Devices Integration](SemanticData/AdoptingCompanionStandardADI.md)
  * [Companion Specification - Information Model for Analyzers](SemanticData/CompanionSpecificationInformationModelFforAnalyzers.md)
  * [ADI Information Model Adoption](SemanticData/ADIInformationModelAdoption.md)
  * [ADI Model Deployment](SemanticData/ADIModelDeployment.md)
  * [Address Space Model Life-cycle](SemanticData/InformationModelLifecycle.md)
  * [Design and Deployment Support](SemanticData/DesignDeploymentSupport.md)
* [`Address Space Management` Implementation](SemanticData/UANodeSetValidation/README.MD)
* [Address Space Prototyping Tool (asp.exe)](SemanticData/AddressSpaceComplianceTestTool/README.md)
  * [`UAModelDesignExport` Library](SemanticData/UAModelDesignExport/README.md)

## Internet of Things (IoT) Archetype

* [Semantic-Data Message Centric Communication](Networking/README.MessageCentricCommunication.md)
* [Internet of Things (IoT) Communication](Networking/README.MD)

## Reactive Communication

* [UA Part 14: PubSub Main Technology Features](Networking/SemanticData/README.PubSubMTF.md)
* [Reactive Networking of Semantic-Data Library](Networking/SemanticData/README.MD)
  * [Underlying Transport over UDP](Networking/UDPMessageHandler/README.md)
  * [Underlying Transport over MQTT](Networking/SemanticData/README_MQTTMapping.md)
  * [Underlying Transport over AMQP](Networking/SemanticData/README_AMQPMapping.md)
  * [Underlying Transport over Ethernet](Networking/SemanticData/README_EthernetMapping.md)
  * [DataSet and Communication Channel Association](Networking/SemanticData/READMEDataDiscoveryAndBinding.md)
  * [Encoding Library](Networking/Encoding/README.md)
* [Getting Started Tutorial](Networking/SemanticData/READMEGettingStartedTutorial.md)
* [Walk-through `ReferenceApplication`](Networking/ReferenceApplication/README.MD)
  * [`ReferenceApplication` Utilities](Networking/ReferenceApplication.Core/README.md)
  * [`ReferenceApplication` Consumer - Data Logger](Networking/DataRepository/DataLogger/README.md)
  * [`ReferenceApplication` Producer - Interoperability Test Data Generator](Networking/SimulatorInteroperabilityTest/README.md)
  * [`ReferenceApplication` Producer - Boilers Set Simulator](Networking/Simulator.Boiler/README.md)

## Configuration

* [Configuration - Executive Summary](Configuration/README.md)
  * [Reactive Networking (RxNetworking) Configuration](Configuration/Networking/README.MD)
  * [DataBinding library](Configuration/DataBindings/README.MD)

## Global Data Discovery

* [Concept](DataDiscovery/README.MD)
* [Domain Model](DataDiscovery/DomainModel.md)

## References

* [See also](REFERENCES.md)
