# Adopting Companion Standard Models - Analyzer Devices Integration

## Introduction

An analyzer is a device comprised of one or more measurement channels, which has its own configuration, status and control. There is a variety of analyzer groups such as light spectrometers, particle size monitoring systems, imaging particle size monitoring systems, acoustic spectrometers, mass spectrometers, chromatographs, imaging systems and nuclear magnetic resonance spectrometers. These groups can be extended and each group can also be further divided.

The main goal of the analyzer device is to provide process data that is generated from scaled data by applying a chemometric model.

Process data is typically represented as a scalar value or a set of scalar values and it is often used for process control. Examples of process data are: concentration, moisture and hardness.

Scaled data is generated from raw data and represents an actual measurement expressed in meaningful units. Scaled data is typically an array of numbers. Examples of scaled data are: absorbance, scatter intensity. To obtain scaled data a mathematical description - analyzer model - of the process and associated information to convert raw data into scaled data is used. Raw Data is generated by an analyzer representing an actual measurement. Raw data is typically represented as an array of numbers. Examples of raw data are: raw spectrum, chromatogram and particle size bin count.

The analyzer configuration is a set of values of all parameters that when set, put the analyzer in a well-defined state.

Analyzers contain measurement channels. A channel is a subset of an analyzer that represents a specific sensing port and associated data, which includes raw and scaled data (e.g. spectrum), configuration, status and control.

To enhance the analyzer behavior or operation replaceable accessories are used. An accessory is a physical device that can be mounted directly on the analyzer or analyzer channel. Examples of accessories are: vial holder, filter wheel, auger, and heater. The accessories are attached using accessory slots.

A sampling point is a physical interface point on the process where the process is monitored. To provide mapping between a channel and a process sampling points the concept of stream is used.

Because there is a large variety of analyzer types from various vendors with many different types of data, including complex arrays and structures, the integration of the analyzers and control and monitoring systems is a real challenge. Initiatives such as Process Analytical Technology are driving analyzer integration and the best way to accomplish this is via open standards. To address the problem two questions can be asked:

- How to get access to (transport) the process data?
- How to represent (model) the process data?

To answer the first question we need a universally accepted, platform-neutral communication standard that allows also addressing the second question, i.e. designing an appropriate information model. OPC Unified Architecture technology meets all the requirements, because:

- It is a platform neutral standard allowing for easy embedded implementation.
- It is designed to support complex data types and object models.
- It is designed to achieve high speed data transfers using efficient binary protocols.
- It has broad industry support beyond just process automation and is being used in support of other industry
standards such as S95, S88, EDDL, MIMOSA, OAGiS.
