# [Object-Oriented Internet](https://github.com/mpostol/OPC-UA-OOI) Partnership Program

## Why follow Object-Oriented Internet

It is said that we are or soon will be citizens of a global village - a world considered as a single community linked by telecommunications. All applications designed atop of network communication can be grouped as follows:

- **human-centric** - applications where the information origin or information destination is an operator
- **machine-centric** - applications where information creation, consumption, networking, and processing are achieved entirely without human interaction

A typical **human-centric** approach is web-service supporting, for example, online bank account management. In this case, it is essential that any uncertainty and necessity to make a decision can be relaxed by human interaction. Coordination of multi-robot behavior in a work-cell or autonomous cars entering a service area fulfills the **machine-centric** scenario. It is crucial that, in this case, any human interaction is impractical or even impossible. **This interoperability scenario requires a machine to machine communication (M2M) demanding multi-vendor devices integration**.

The human-centric global village is almost done. However, the machine-centric global village still needs design and development effort. Information and Communication Technology (ICT) has provided society with a vast variety of distributed machine-oriented applications including the meaningful Machine to Machine (M2M) communication targeting distributed mobile applications in the context of new emerging disciplines, i.e. Industry 4.0 (I40) and Internet of Things (IoT). However, it is a real challenge if the mentioned machines are provided by a vast variety of vendors. The real challenge we are facing is how to produce independently smart things (i.e. machines, devices, appliances, assets, etc.) to guarantee that they are plug and produce ready. There are no doubts, it requires standardization. I believe that while producing the machines in compliance with the OPC UA this issue is relaxed by applying the following OPC UA standardized concepts:

- **Information Model** - all about how to design a formal but mutually meaningful and shareable description of the considered process
- **Address Space** - all about how to instantiate and expose to the network a life replica of the process providing real-time data according to the above-mentioned formal description

The standardization process may be "paper-driven" or "community-driven". In both cases, standardization is indispensable but not sufficient. Let me recall that the foundation for the human-centric global village is just the Internet Protocol defined many years ago and derived from the university intellectual properties published as an open-access document (RFC "paper").

The open-access [**Object-Oriented Internet** (OOI)](https://commsvr.gitbook.io/ooi/readme) umbrella project targets multi-vendor plug-and-produce machines interoperability scenarios targeting all aspects of the machine-centric global village concept aimed at providing reusable deliverables, training, best practice rules, prototyping, compliance testing and dissemination of valuable results.

Concluding, to make real progress, I propose to focus on leveraging:

- well-known concepts, e.g. **Object-Oriented Programming**, Service-Oriented Architecture, Data-Oriented Architecture (DOA), cloud computing, etc.
- existing and emerging standards, e.g. OPC UA, AMQP, MQTT, etc.
- converged academic knowledge and practitioners experience
- available human-centric global village interoperability platforms, e.g. [GitHub](https://github.com/mpostol), [Wordpress](https://mpostol.wordpress.com), [Research Gate](https://www.researchgate.net/profile/Mariusz_Postol), [LinkedIn](https://www.linkedin.com/in/mpostol), [Twitter](https://twitter.com/mpostol), [Gitter](https://gitter.im/mpostol/OPC-UA-OOI), etc.
- loosely coupled research and development team of direct contributors

and contribute to the following call to actions

- brainstorm ideas
- collect requirements
- collect existing solutions and deliverables
- provide proof of concept
- provide deliverables
- build up a loosely coupled team

## Why follow me

I have 35+ years of experience in designing and deploying highly distributed applications having managed 100+ innovative projects for industry including aviation, heat engineering, power engineering, and mining. I am the author of

- **Process Observer** concept and implementation
- **CommServer** OPC based communication software family for the management and optimization of data transfer
- **SmartFactory** workflow management system

For 15 years I have been OPC Foundation active member involved in a vast variety of projects related to the OPC Unified Architecture.

I am engaged in many research projects as a university teacher and scientist. I am the author of 40+ publications, lectures, presentations and training sessions. I have a degree as a Master Engineer in Electrical Engineering from the Technical University of Lodz and a Ph.D. in Process Control Engineering and Robotics.

I am the founder and Executive Director of CAS.

## How to be rewarded

As a result of participation in the sponsorship program, direct contribution or using the deliverables as the end user you

- can directly influence further development of the OOI prioritizing the tasks backlog
- will access OOI documentation and deliverables
- may use the deliverables in commercial products
- may distribute the deliverables including source code
- will have opportunities to join working groups, online discussion groups, and collaborations for new technology initiatives
- will have access to and direct contribution to new publications
- will make use of the OOI logo
- will have the opportunity to list your products in the online product guide
- will have the opportunity to announce new solutions and products online

## How to be involved

I am a researcher who is passionate about applying knowledge and experience in building a machine-centric global village. Let's build it with you and for you. To be involved you may

- join the Sponsorship Program
- directly contribute
- adopt deliverables

### Sponsorship Program

You can sponsor the OOI using my profiles

- [**GitHub**](https://github.com/sponsors/mpostol) anyone with a GitHub account can sponsor anyone with a sponsored developer profile through a recurring monthly payment. You can choose from multiple sponsorship tiers, with monthly payment amounts and benefits that are set. For more information, see [About GitHub Sponsors](https://help.github.com/en/github/supporting-the-open-source-community-with-github-sponsors/about-github-sponsors)

- [**Patreon**](https://www.patreon.com/mpostol) - Patrons are the keepers of the cash, the power people with the credit/debit cards. A patron will kick-off the chain of events by typing in credit/debit card info, or syncing a payment account, like PayPal. A more detailed description is covered by the [Payment Glossary - payment processing on Patreon](https://support.patreon.com/hc/en-us/articles/360024774831-Payment-Glossary-payment-processing-on-Patreon).

Choose the profile based on your convenience, but it is worth stressing that there is [GitHub Sponsors Matching Fund](https://help.github.com/en/github/supporting-the-open-source-community-with-github-sponsors/about-github-sponsors#about-the-github-sponsors-matching-fund).

Regardless of which profile you choose the same tiers can be selected. The detailed tiers description is [here](Tiers.md).

### Contribute

Directly contribute as an active member of the OOI team [CONTRIBUTING](https://github.com/mpostol/OPC-UA-OOI/blob/master/CONTRIBUTING.md). Toi get started you may use the guide [The beginner's guide to contributing to a GitHub project](https://akrabat.com/the-beginners-guide-to-contributing-to-a-github-project/)

### Adopt Deliverables

All OOI repositories on GitHub are used to share open-source software and documentation. These repositories are truly open source because they are licensed under the terms of the MIT license, therefore, others are free to use, change, and distribute the software. I hope that all end-users adopting the software will provide reciprocal feedback by email, on [Gitter](https://gitter.im/mpostol/OPC-UA-OOI), reporting feature requests/issues, etc.
